import dendropy as dpy
from geopy import distance
import re
import argparse
import os
import numpy as np
import matplotlib.pyplot as plt

locval_re = re.compile(r'[0-9]+\.[0-9E\-]+')


def calc_velocity(node1, node2):
    geo_dist = distance.distance((node1.lat,node1.long) , (node2.lat,node2.long)).km
    time = node1.height - node2.height
    velo = geo_dist/time

    return velo

def parse_beast_tree_node_info(tree):

    for nd in tree.postorder_node_iter():

        beast_info = {}

        node_comment = nd._get_annotations()
        for annot in node_comment:
            if str(annot).startswith('location1='):
                loc1_val = float(locval_re.search(str(annot)).group())
                nd.lat = loc1_val
            if str(annot).startswith('location2='):
                loc2_val = float(locval_re.search(str(annot)).group())
                nd.long = loc2_val
            if str(annot).startswith('height='):
                height_val =  float(locval_re.search(str(annot)).group())
                nd.height = height_val

    return tree

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("-input", "--input_file", type=str,
                        help="Tree file in nexus format obtained from TreeAnnotator", required=True)

    args = parser.parse_args()
    out_name = os.path.splitext(args.input_file)[0] + '_velo.nex'
    out_name_velo = os.path.splitext(args.input_file)[0] + '_velo.txt'
    fig_name = os.path.splitext(args.input_file)[0] + '_velo_distribution.png'

    tree = dpy.Tree.get_from_path(args.input_file, 'nexus')
    tree = parse_beast_tree_node_info(tree)

    velocities = []
    for node in tree.postorder_node_iter():
        parent = node.parent_node
        if parent != None:
            velo = calc_velocity(parent, node)
            node.velo = velo
            node.annotations.add_new(name='velocity', value=str(velo))
            if velo > 0:
                velocities.append(velo)
    out_velo = open(out_name_velo, 'w')
    out_velo.write(" ".join([str(x) for x in velocities]))
    out_velo.close()

    velocities = np.array(velocities)
    fig, axs = plt.subplots(1, 1, 
                        figsize =(10, 7),  
                        tight_layout = True)
    axs.hist(velocities, bins = 50)
    axs.set_title('Distribution of velocities')
    axs.set_xlabel('Velocity, km/year')
    fig.savefig(fig_name, bbox_inches='tight', dpi=300)
    plt.show()
    
    tree.write(path=out_name, schema="nexus")
    